unit RegHelpers;

{$RTTI EXPLICIT METHODS([]) PROPERTIES([]) FIELDS([])}

interface

uses
  Registry, Windows;

function WriteStrToReg(key: HKEY; Path: string; rk: String;
  str: String): Boolean;
function ReadStrFromReg(key: HKEY; Path: string; rk: String; var str: String)
  : Boolean; overload;
function ReadStringFromReg(key: HKEY; Path: string; rk: String; def: String)
  : string; overload;
function ReadStrFromReg(Reg: TRegistry; Name: string; Def: String)
  : String; overload;
function ReadIntFromReg(Reg: TRegistry; Name: string; Def: Integer): Integer;
function CreateStrToReg(key: HKEY; Path: string; rk: String;
  str: String): Boolean;
function DeleteFromReg(key: HKEY; Path: string; rk: String): Boolean;
function WriteBinaryToReg(key: HKEY; Path: string; rk: String;
  str: String; createIfNotExist:boolean=true): Boolean;
function WriteIntToReg(key: HKEY; Path: string; rk: String;
  int: integer; check64:boolean=true): Boolean;
function deleteRegistry(key:string; root:HKEY=0):boolean; overload;
function saveregistry(key,value,data:string; root:HKEY=0):boolean;
function ReadStrFromRegx64(key: HKEY; Path: string; rk: String; var str: String)
  : Boolean; overload;

implementation

uses
  SysUtils, System.Classes, System.StrUtils, Vcl.Dialogs{, SharedGlobal};

function loadregistry(key,value:string; root:HKEY=0):string;
begin
result:='';
with Tregistry.create do
  try
    try
      if root > 0 then rootKey:=root;
      if openKey(key, FALSE) then
        begin
        result:=readString(value);
        closeKey();
        end;
    finally free end
  except end
end; // loadregistry

function saveregistry(key,value,data:string; root:HKEY=0):boolean;
begin
result:=FALSE;
with Tregistry.create do
  try
    if root > 0 then rootKey:=root;
    try
      createKey(key);
      if openKey(key, FALSE) then
        begin
        WriteString(value,data);
        closeKey;
        result:=TRUE;
        end;
    finally free end
  except end;
end; // saveregistry

function deleteRegistry(key,value:string; root:HKEY=0):boolean; overload;
var
  reg:Tregistry;
begin
reg:=Tregistry.create;
if root > 0 then reg.RootKey:=root;
result:=reg.OpenKey(key,FALSE) and reg.DeleteValue(value);
reg.free
end; // deleteRegistry

function deleteRegistry(key:string; root:HKEY=0):boolean; overload;
var
  reg:Tregistry;
  ss:TstringList;
  i:integer;
  deleteIt:boolean;
begin
reg:=Tregistry.create;
if root > 0 then reg.RootKey:=root;
result:=reg.DeleteKey(key);
// delete also parent keys, if empty
ss:=Tstringlist.create;
while key>'' do
  begin
  i:=LastDelimiter('\',key);
  if i=0 then break;
  setlength(key, i-1);
  if not reg.OpenKeyReadOnly(key) then break;
  reg.GetValueNames(ss);
  deleteIt:=(ss.count=0) and not reg.HasSubKeys;
  reg.CloseKey;
  if deleteit then reg.deleteKey(key)
  else break;
  end;
ss.free;
reg.free
end; // deleteRegistry

function WriteIntToReg(key: HKEY; Path: string; rk: String;
  int: integer; check64:boolean=true): Boolean;
var
  Reg: TRegistry;
begin
  if check64 then begin
    Reg := TRegistry.Create(KEY_WRITE or KEY_WOW64_64KEY);
  end else begin
    Reg := TRegistry.Create;
  end;
  Result := False;

  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    Reg.WriteInteger(rk, int);
    Result := True;
  except
    {on e:exception do
      showmessage(e.Message);}
  end;

  FreeAndNil(Reg);
end;

function WriteStrToReg(key: HKEY; Path: string; rk: String;
  str: String): Boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := False;

  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    Reg.WriteString(rk, str);
    Result := True;
  except
  end;

  FreeAndNil(Reg);
end;


function ReadStrFromReg(Reg: TRegistry; Name: string; Def: String): String;
begin
  Result := Def;

  if Reg.ValueExists(Name) then
    Result := Reg.ReadString(Name)
end;

function ReadStrFromReg(key: HKEY; Path: string; rk: String;
  var str: String): Boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := False;

  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    str := ReadStrFromReg(Reg, rk, '');
    Result := True;
  except
    str := '';
  end;

  FreeAndNil(Reg);
end;

function ReadStrFromRegX64(key: HKEY; Path: string; rk: String;
  var str: String): Boolean;
var
  Reg: TRegistry;
  I: Integer;
begin
  Result := False;
  for I := 1 to 3 do begin
    try
      case i of
        1:Reg := TRegistry.Create;
        2:Reg := TRegistry.Create(KEY_READ or KEY_WOW64_32KEY);
        3:Reg := TRegistry.Create(KEY_READ or KEY_WOW64_64KEY);
      end;
      Reg.RootKey := key;
      if Reg.OpenKey(Path, False) then begin
        if Reg.ValueExists(rk) then begin
          str := Reg.ReadString(rk);
          Result := True;
        end;
      end;
    except
      str := '';
    end;
    FreeAndNil(Reg);
    if result then break;
  end;
end;

function ReadStringFromReg(key: HKEY; Path: string; rk: String; def: String)
  : string; overload;
var
  Reg: TRegistry;
begin
  Result := def;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    result := ReadStrFromReg(Reg, rk, def);
  finally
    FreeAndNil(Reg);
  end;
end;

function ReadIntFromReg(Reg: TRegistry; Name: string; Def: Integer): Integer;
begin
  Result := Def;

  if Reg.ValueExists(Name) then
    Result := Reg.ReadInteger(Name)
end;

function CreateStrToReg(key: HKEY; Path: string; rk: String;
  str: String): Boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := False;

  try
    Reg.RootKey := key;
    if Reg.OpenKey(Path, True) then begin
      Reg.WriteString(rk, str);
      Result := True;
    end;
  except
  end;

  FreeAndNil(Reg);
end;

function DeleteFromReg(key: HKEY; Path: string; rk: String): Boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
  Result := False;
  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    Reg.DeleteValue(rk);
    Result := True;
  except
  end;
  FreeAndNil(Reg);
  Reg := TRegistry.Create(KEY_WRITE OR KEY_WOW64_32KEY);
  //Result := False;
  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    Reg.DeleteValue(rk);
    Result := True;
  except
  end;
  FreeAndNil(Reg);
  Reg := TRegistry.Create(KEY_WRITE);
  //Result := False;
  try
    Reg.RootKey := key;
    Reg.OpenKey(Path, False);
    Reg.DeleteValue(rk);
    Result := True;
  except
  end;
  FreeAndNil(Reg);
end;

function WriteBinaryToReg(key: HKEY; Path: string; rk: String;
  str: String; createIfNotExist:boolean=true): Boolean;
var
  Buf: array of Byte;
  Reg: TRegistry;
  I, Len: Integer;
  strList:TStringList;
begin
  str:=replaceStr(str,Slinebreak,'');
  str:=replaceStr(str,'\','');
  strList:=TStringList.Create;
  strList.Delimiter:=',';
  strList.DelimitedText:=str;
  Len := strList.Count;
  SetLength(Buf, Len);
  for I := 0 to strList.Count-1 do begin
    Buf[i]:=StrToInt('$' + trim(strList[i]));
  end;

  Reg := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
  Result := False;

  try
    Reg.RootKey := key;
    if Reg.OpenKey(Path, createIfNotExist) then begin
      Reg.WriteBinaryData(rk, buf[0],len);
      Result := True;
    end;
  except
  end;

  FreeAndNil(Reg);
  strList.free;
end;

end.
